package com.example.selenium.demo.Service;

import org.openqa.selenium.chrome.ChromeOptions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
@Service
public class ChromeSeleniumDriverPool {

    private static List<ModifiedChromeDriver> chromeDriverPool = new ArrayList<>();

    private static Logger logger= LoggerFactory.getLogger(ChromeSeleniumDriverPool.class);

    private static String defaultChromedrivePath="C:\\Program Files (x86)\\Google\\Chrome\\Application\\chromedriver.exe";

    private static  int defaultPoolSizeKey=1;

    private  static String defaultDriver="webdriver.chrome.driver";

    private static Boolean defaultHeadless=false;

    static {
        ChromeOptions chromeOptions;
        Properties properties=new Properties();

        String driverPath = null;
        int poolSize = 1;
        String driver=null;
        Boolean isHeadless=true;

        driverPath=defaultChromedrivePath;
        poolSize=defaultPoolSizeKey;
        driver=defaultDriver;
        isHeadless=defaultHeadless;

        logger.info(String.format("初始化参数：driver=%s ,poolsize=%s ,path=%s , headless=%s",
                driver,poolSize,driverPath,isHeadless));

        System.setProperty(driver, driverPath);
        chromeOptions = new ChromeOptions();
        if (isHeadless){
            chromeOptions.addArguments("--no-sandbox");
            chromeOptions.addArguments("--disable-dev-shm-usage");
            chromeOptions.addArguments("--headless");
            chromeOptions.addArguments("--blink-settings=imagesEnabled=false");
        }

//        设置浏览器窗口打开大小  （非必须）
        chromeOptions.addArguments("--window-size=1920,1080");

        try {
            initPool(chromeOptions,poolSize);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    /**
     * 初始化驱动对象池
     * @param chromeOptions
     * @param size
     */
    private synchronized static void initPool(ChromeOptions chromeOptions,int size) throws InterruptedException {
        if (chromeDriverPool.size()>0){
            for (ModifiedChromeDriver modifiedChromeDriver:chromeDriverPool){
                try {
                    modifiedChromeDriver.quit();
                }catch (Exception ignored){
                }
            }
        }
        chromeDriverPool.clear();

        for (int i=0;i<size;i++){
            ModifiedChromeDriver chromeDriver = new ModifiedChromeDriver(chromeOptions);
            chromeDriverPool.add(chromeDriver);
        }
    }

    /**
     * 从驱动池中获取驱动对象
     * @return
     */
    public synchronized static ModifiedChromeDriver getDriver(){
        while (true) {
            for (ModifiedChromeDriver modifiedChromeDriver : chromeDriverPool) {
                if (!modifiedChromeDriver.isLock()){
                    modifiedChromeDriver.lock();

                    System.out.println("当前获取的浏览器实例的下标："+chromeDriverPool.indexOf(modifiedChromeDriver));

                    return modifiedChromeDriver;
                }
            }
            try {
                Thread.sleep(50);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }


}
